#!/bin/sh

##################### FUNCTIONS ##########################

prompt()
{
	if test $NONINTERACTIVE; then
		return 0
	fi

	# BASH echo -ne "$*" >&2
	echo -n "$*" >&2
	read CMD rest
	return 0
}

# ----------------------------------------------------------------------------
# Get Yes/No
# ----------------------------------------------------------------------------
getyn()
{
	if test $NONINTERACTIVE; then
		return 0
	fi

	while prompt "$* (y/n) "
	do	case $CMD in
			[yY])	return 0
				;;
			[nN])	return 1
				;;
			*)	echo -e "\nPlease answer y or n" >&2
				;;
		esac
	done
}           


usage()
{
	echo
 	echo "Usage info"
	echo
	echo "./release.sh rev 1 1 0 0 tag [y|n]"
	echo

	exit 1
}


##################### MAIN ##########################


ROOT_DIR=$(pwd)

OS_TYPE=`uname -o`

if [ $OS_TYPE != "Cygwin" ]; then
	OS_TYPE="linux"
	OSLIB=so
else
	OSLIB=dll
fi

SNG_VER_FILE=server/libsngmp/src/sng_tc_version.h

ver=`svnversion`
ver=${ver// /}
ver=${ver//M/}

rev=""
major=""
minor=""
patch=""
tag=""

while [ 1 ]
do
    if [ "$1" = "" ]; then
		break;
	fi

	if [ "$1" = "rev" ]; then
      	major=$2
		shift
      	minor=$2
		shift
      	rev=$2
		shift
      	patch=$2
		shift
	fi
	if [ "$1" = "tag" ]; then
		if [ "$2" = "" ]; then
			usage
		fi
      	tag=$2
		shift
	fi

	shift

done

arch=`uname -m`
echo $arch | grep "i.86" > /dev/null 2> /dev/null
if [ $? -eq 0 ]; then
  	arch='i686'
else
	arch='x86_64'
fi   

echo
echo
echo "Build Info"
echo "   Major=$major"
echo "   Minor=$minor"
echo "   Rev  =$rev"
echo "   Patch=$patch"
echo "   Tag  =$tag"
echo "   Arch =$arch"
echo       

if [ "$major" = "" ] || [ "$minor" = "" ] || [ "$rev" = "" ] || [ "$patch" = "" ] || [ "$tag" = "" ]; then
	usage
	exit 1
fi

if [ "$patch" = "tag" ]; then
	echo
	echo "Error: Invalid patch version: set to $patch"
	echo
  	usage
	exit 1
fi


if [ $OS_TYPE = "linux" ]; then
	if [ $patch -eq 0 ]; then
		rel_name="brdcom-linux-tg3-api-"$major"."$minor"."$rev"."$arch
	else
		rel_name="brdcom-linux-tg3-api-"$major"."$minor"."$rev"."$patch"."$arch
	fi
else
	if [ $patch -eq 0 ]; then
		rel_name="brdcom-windows-"$arch"-"$major"."$minor"."$rev
	else
		rel_name="sngmp-windows-"$arch"-"$major"."$minor"."$rev"."$patch
	fi
fi



clear
          
echo
echo
echo "Build Info"
echo "   Major=$major"
echo "   Minor=$minor"
echo "   Rev  =$rev"
echo "   Patch=$patch"
echo "   Tag  =$tag"
echo "   Arch =$arch"
echo

echo
getyn "Build Release For: $rel_name - Tag $tag - SVN $ver "
if [ $? -ne 0 ]; then
  	exit 1
fi
echo


echo
echo
echo "Building Version = $rel_name  SVN Rev=$ver  Arch=$arch"
echo


if [ $OS_TYPE = "linux" ]; then
	# clean everything
	make clean
	make -C driver clean
else
	echo "Not supported yet"
	exit 1
fi

rm -rf releases/$rel_name
mkdir -p releases/$rel_name/
tar cfz rel-$$.tgz * --exclude=releases
mv rel-$$.tgz releases/$rel_name/

cd releases/$rel_name/
tar xfz rel-$$.tgz
rm -f rel-$$.tgz

find . -name '*.svn' | xargs rm -rf
find . -name '.*.o.cmd'  | xargs rm -f
find . -name '*.*~'  | xargs rm -f
find . -name 'diff'  | xargs rm -f
find . -name '*.swp' | xargs rm -f 
find . | xargs touch -t 200808010900
cd ../

tar cfz $rel_name".tgz" $rel_name

cd $ROOT_DIR

echo
echo "Done"

