
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <signal.h>
#include <net/if.h>
#ifndef __GLIBC__
#include <linux/if_arp.h>
#include <linux/if_ether.h>
#endif
#include <linux/sockios.h>
#include <linux/mii.h>

#include "tg3_iface.h"
#include "tg_test.h"

int skfd;
static struct ifreq ifr;  

int MakeConnection(char *i_name )
{
	errno = 0;
	skfd = socket(AF_INET, SOCK_DGRAM,0);
   	if( skfd < 0 ) {
      		perror("Socket");
      		return -1;
   	} /* if */

	strncpy(ifr.ifr_name, i_name, IFNAMSIZ);

	if (ioctl(skfd, SIOCGMIIPHY, &ifr) < 0) {
		 fprintf(stderr, "SIOCGMIIPHY on '%s' failed: %s\n",
		    i_name, strerror(errno));
		return -1;
	}

	printf("Socket bound to %s\n\n",i_name);

	return 0;
}                          


static int tg3_read(int skfd, int location, uint32_t *val)
{
    struct tg3_priv_ioctl_data *data = (struct tg3_priv_ioctl_data *)&ifr.ifr_ifru;
	memset(data,0,sizeof(*data));
    data->reg_num = location;
    if (ioctl(skfd, SIOC_TG3_REG_READ, &ifr) < 0) {
		fprintf(stderr, "SIOC_TG3_REG_READ on %s failed: %s\n", ifr.ifr_name,
			strerror(errno));
		return -1;
    }

	*val=data->val;

    return 0;
}

static int tg3_write(int skfd, int location, int value)
{
	int err;
    struct tg3_priv_ioctl_data *data = (struct tg3_priv_ioctl_data *)&ifr.ifr_ifru;
	memset(data,0,sizeof(*data));
    data->reg_num = location;
    data->val = value;

	data->flush=0;
	data->flush_timeout=0;

	err=ioctl(skfd, SIOC_TG3_REG_WRITE, &ifr);
	if (err < 0) {
		fprintf(stderr, "SIOC_TG3_REG_WRITE on %s failed: %s\n", ifr.ifr_name,
			strerror(errno));
    }
	return err;
}

static int tg3_write_flush(int skfd, int location, int value)
{
	int err;
    struct tg3_priv_ioctl_data *data = (struct tg3_priv_ioctl_data *)&ifr.ifr_ifru;
	memset(data,0,sizeof(*data));
    data->reg_num = location;
    data->val = value;

	data->flush=1;
	data->flush_timeout=0;

	err=ioctl(skfd, SIOC_TG3_REG_WRITE, &ifr);
	if (err < 0) {
		fprintf(stderr, "SIOC_TG3_REG_WRITE on %s failed: %s\n", ifr.ifr_name,
			strerror(errno));
    }
	return err;
}
static int tg3_write_flush_t(int skfd, int location, int value, int ms)
{
	int err;
    struct tg3_priv_ioctl_data *data = (struct tg3_priv_ioctl_data *)&ifr.ifr_ifru;
	memset(data,0,sizeof(*data));
    data->reg_num = location;
    data->val = value;

	if (ms <= 0) {
		return -1;
	}

	data->flush=1;
	data->flush_timeout=ms;

	err=ioctl(skfd, SIOC_TG3_REG_WRITE, &ifr);
	if (err < 0) {
		fprintf(stderr, "SIOC_TG3_REG_WRITE on %s failed: %s\n", ifr.ifr_name,
			strerror(errno));
    }
	return err;
}

////////////////////////////
////////////////////////////
#define SPI_CLK_OUT_BIT 	(1 << 16)
#define SPI_SDO_OUT_BIT 	(1 << 14)
#define SPI_CLK_OUT_EN_BIT 	(1 << 13)
#define SPI_SDO_OUT_EN_BIT  (1 << 11)
#define SPI_SDI_IN_BIT		(1 << 9)

#define SPI_TIMEOUT_MS 10

//#define FAST_SPI

#ifdef FAST_SPI
#define SPI_SHORT_DELAY {}
#else
#define SPI_SHORT_DELAY {usleep(1);}
#endif
	
#define SPI_SYNC_DELAY  {usleep(300000);}
#define SPI_DATA_DELAY 	{usleep(50);}
#define SPI_BUSY_TIMEOUT_MS 100
#define DELAY_SWITCH_ACCESS {usleep(20000);}
#define DELAY_MCU_ACCESS {usleep(100);}


#define SUCCESS 0
#define IND_DEVICE_ID		0x55AA
#define MCU_DEVICE_ID		0x1234
#define SWITCH_DEVICE_ID	0x00003118

#define SPI_MAGIC_NUM 0x5A5A
#define SPI_RD_BIT 			(1 << 15)
#define REG_TARGET_DONE_BIT		(1 << 15)

#define MCU_TARGET 	(1 << 0)
#define SWITCH_TARGET 	(0 << 0)

#define READ_OP	(1 << 1)
#define WRITE_OP	(0 << 1)

#define ERR_SPI_OFFSET 0
#define ERR_IND_DEV_ID		(ERR_SPI_OFFSET + 1)
#define ERR_MCU_DEV_ID		(ERR_SPI_OFFSET + 2)
#define ERR_SWITCH_DEV_ID	(ERR_SPI_OFFSET + 3)
#define ERR_SPI_TIMEOUT		(ERR_SPI_OFFSET + 4)
#define ERR_SPI_VAL	   	(ERR_SPI_OFFSET + 5)
#define ERR_SWITCH_READBACK   	(ERR_SPI_OFFSET + 6)




typedef enum{
IND_ID_REG_ADDR=0,
IND_RESULT_ADDR,
IND_TARGET_ADDR,
IND_NUM_BYTES,
IND_PAGE_ADDR,
IND_REG_ADDR,
IND_DATA,

NUM_INDIRECT_REGISTERS
}INDIRECT_REGISTERS;


/////////////////////////////////
/////////////////////////////////
//MCU REGISTER STUFF
typedef enum{
DEVICE_ID_ADDR=0,
BOARD_REV_ADDR,
FIRMWARE_VERSION_ADDR,
BOARD_TEMP_C_ADDR,
FAN_SPEED_ADDR,
RESET_PINS_ADDR,
VOC_RELOAD_PINS_ADDR,
SWITCH_CONTROL_ADDR,

NUM_MCU_REGS
}MCU_REG_MAP;


int init_spi_pins(void);
int read_mcu_reg(int address, int *data);
int read_switch_reg(int num_bytes, int page_addr, int reg_addr, int *data);
int write_mcu_reg(int address, int data);
int write_switch_reg(int num_bytes, int page_addr, int reg_addr, int data);
int play_reset(void);
int play_reload(void);
int play_switch(void);
int read_loop(void);


static int spi_clock_out_16(int out_data, int *in_data);
static int spi_ind_reg_write(int address, int data);
static int spi_ind_reg_read(int address, int *data);
static int wait_for_done_bit(void);

////////////////////////////
////////////////////////////
//Configures the pins used for the spi interface, must be called prior
//to communicating with the mcu.
int init_spi_pins(void)
{   int err, reg_data, low_16, high_16;
	uint32_t data;

	err=tg3_read(skfd,GRC_LOCAL_CTRL,&data);
	if (err < 0) {
		printf("Failed to read MII reg 0x%0X\n",GRC_LOCAL_CTRL);
		return -1;
	}

	//Set the output enable bits for SPI_CLK and SPI_SDO, and clear the outputs
	data |= (SPI_CLK_OUT_EN_BIT | SPI_SDO_OUT_EN_BIT);
	data &= ~(SPI_CLK_OUT_BIT | SPI_SDO_OUT_BIT);
	err=tg3_write_flush_t(skfd, GRC_LOCAL_CTRL, data, SPI_TIMEOUT_MS);
	if (err < 0) 
	   	return err;
	
	
	//Clock out byte to synchronize mcu spi port
	err = spi_clock_out_16(0, &reg_data);
	if(err != SUCCESS)
	 	return err;
    
	SPI_SYNC_DELAY

	//Read ID reg to make sure value is correct
	err = spi_ind_reg_read(IND_ID_REG_ADDR, &reg_data);
	if(err != SUCCESS)
		return err;

	if(reg_data != IND_DEVICE_ID)
		return ERR_IND_DEV_ID;


	//read mcu ID reg to make sure value is correct
	err = read_mcu_reg(DEVICE_ID_ADDR, &reg_data);
	if(err != SUCCESS)
		return err;


	if(reg_data != MCU_DEVICE_ID)
	{	//printf("reg_data = 0x%x.\n", reg_data);
		return ERR_MCU_DEV_ID;
	}
	
	//check Switch access
	err = read_switch_reg(2, 2, 0x30, &low_16);
	if(err != SUCCESS)
		return err;

	err = read_switch_reg(2,2,0x32, &high_16);
	if(err != SUCCESS)
		return err;

	low_16 |= (high_16 << 16);
	if(low_16 != SWITCH_DEVICE_ID)
		return ERR_SWITCH_DEV_ID;


	return SUCCESS;	
}



 
////////////////////////////
////////////////////////////
//Clocks out_data on the SDO pin and at the same time clocks in data on 
//the SDI pin. All data is MSB first.
static int spi_clock_out_16(int out_data, int *in_data)
{	int x, err, in;
 	uint32_t temp;

    in = 0;
	for(x = 0; x < 16; x++)
	{	in <<= 1;


		//Read register and change the SDO bit
		err=tg3_read(skfd,GRC_LOCAL_CTRL,&temp);
		if (err < 0) 
			return err;
		
		if(out_data & 0x8000)
           		temp |= SPI_SDO_OUT_BIT;
		else
			temp &= (~SPI_SDO_OUT_BIT);


		//Write the register back
		err=tg3_write_flush_t(skfd, GRC_LOCAL_CTRL, temp, SPI_TIMEOUT_MS);
		if (err < 0)
	   		return err;
	
      	
		SPI_SHORT_DELAY

		//clock high
		temp |= SPI_CLK_OUT_BIT;
		err=tg3_write_flush_t(skfd, GRC_LOCAL_CTRL, temp, SPI_TIMEOUT_MS);
		if (err < 0)
			return err;

        
		SPI_SHORT_DELAY
		
		//Read the SDI pin
		err=tg3_read(skfd,GRC_LOCAL_CTRL,&temp);
		if (err < 0) 
			return err;
		
		if(temp & SPI_SDI_IN_BIT)
			in |= 1;
        

		//clock low
		temp &= ~SPI_CLK_OUT_BIT;
	    err=tg3_write_flush_t(skfd, GRC_LOCAL_CTRL, temp, SPI_TIMEOUT_MS);
		if (err < 0)
			return err;
		
		
		SPI_SHORT_DELAY

		out_data <<= 1;
	}
	*in_data = in;
	return SUCCESS;
	
}


////////////////////////////
////////////////////////////
//PRECONDITION: init_spi_pins must be called.
//Writes indirect register.
static int spi_ind_reg_write(int address, int data)
{	int err,not_used;

	err = spi_clock_out_16(SPI_MAGIC_NUM,&not_used);
	if(err != SUCCESS)
		return err;
	
	SPI_DATA_DELAY
	
	err = spi_clock_out_16(address & (~SPI_RD_BIT),&not_used);
	if(err != SUCCESS)
		return err;
		
	SPI_DATA_DELAY
	
	err = spi_clock_out_16(data,&not_used);
	if(err != SUCCESS)
		return err;
		
	SPI_DATA_DELAY
	return SUCCESS;
}


////////////////////////////
////////////////////////////
//PRECONDITION: init_spi_pins must be called.
//Reads indirect register.
static int spi_ind_reg_read(int address, int *data)
{	int err;

	err = spi_clock_out_16(SPI_MAGIC_NUM,data);
	if(err != SUCCESS)
		return err;
		
	SPI_DATA_DELAY
	
	err = spi_clock_out_16(SPI_RD_BIT | address,data);
	if(err != SUCCESS)
		return err;
		
	SPI_DATA_DELAY
	
	err = spi_clock_out_16(0,data);
	if(err != SUCCESS)
		return err;
		
	SPI_DATA_DELAY
	return SUCCESS;
}



////////////////////////////
////////////////////////////
static int wait_for_done_bit(void)
{	int err, data, timeout_ms = SPI_BUSY_TIMEOUT_MS;

	//make sure mcu not busy
	while(timeout_ms-- != 0)
	{	err = spi_ind_reg_read(IND_RESULT_ADDR, &data);
    		if(err != SUCCESS)
			return err;
		
		if(data & REG_TARGET_DONE_BIT);
			break;

		usleep(1000);
	}

	if(timeout_ms == 0)
		return ERR_SPI_TIMEOUT;
	else if((data & (~REG_TARGET_DONE_BIT)) != SUCCESS)  //done bit set but return value != SUCCESS
		return ERR_SPI_VAL;
	else
		return SUCCESS;
} 

////////////////////////////
////////////////////////////
//Reads a register from microcontroller
int read_mcu_reg(int address, int *data)
{	int err;

	//make sure interface not busy
	err = wait_for_done_bit();
	if(err != SUCCESS)
		return err;

	//Write the address
	err = spi_ind_reg_write(IND_REG_ADDR, address);
	if(err != SUCCESS)
		return err;
	
	//Write the target register selecting MCU as the target
	err = spi_ind_reg_write(IND_TARGET_ADDR, MCU_TARGET | READ_OP);
	if(err != SUCCESS)
		return err;
	
	DELAY_MCU_ACCESS
	
	//wait for done bit to be set
	err = wait_for_done_bit();
	if(err != SUCCESS)
		return err;
	
	//read the data reg
	err = spi_ind_reg_read(IND_DATA, data);
	if(err != SUCCESS)
		return err;

    	
	return SUCCESS;
}


////////////////////////////
////////////////////////////
//Write a microcontroller register
int write_mcu_reg(int address, int data)
{	int err;

	//make sure interface not busy
	err = wait_for_done_bit();
	if(err != SUCCESS)
		return err;

	//Write the address
	err = spi_ind_reg_write(IND_REG_ADDR, address);
	if(err != SUCCESS)
		return err;
	
	//Write the data reg
	err = spi_ind_reg_write(IND_DATA, data);
	if(err != SUCCESS)
		return err;


	//Write the target register selecting MCU as the target
	err = spi_ind_reg_write(IND_TARGET_ADDR, MCU_TARGET | WRITE_OP);
	if(err != SUCCESS)
		return err;
	
	DELAY_MCU_ACCESS
	
	//wait for done bit to be set
	err = wait_for_done_bit();
	if(err != SUCCESS)
		return err;
	
	
	return SUCCESS;
}


////////////////////////////
////////////////////////////
//Reads a register from microcontroller
int read_switch_reg(int num_bytes, int page_addr, int reg_addr, int *data)
{	int err;

	//make sure interface not busy
	err = wait_for_done_bit();
	if(err != SUCCESS)
		return err;

	//Write the num_bytes
	err = spi_ind_reg_write(IND_NUM_BYTES, num_bytes);
	if(err != SUCCESS)
		return err;


	//Write the page address
	err = spi_ind_reg_write(IND_PAGE_ADDR, page_addr);
	if(err != SUCCESS)
		return err;
	
	//Write the reg address
	err = spi_ind_reg_write(IND_REG_ADDR, reg_addr);
	if(err != SUCCESS)
		return err;


	//Write the target register selecting MCU as the target
	err = spi_ind_reg_write(IND_TARGET_ADDR, SWITCH_TARGET | READ_OP);
	if(err != SUCCESS)
		return err;
	

	DELAY_SWITCH_ACCESS
	
	
	//wait for done bit to be set
	err = wait_for_done_bit();
	if(err != SUCCESS)
		return err;
	
	//read the data reg
	err = spi_ind_reg_read(IND_DATA, data);
	if(err != SUCCESS)
		return err;

    	
	return SUCCESS;
}

////////////////////////////
////////////////////////////
//Reads a register from microcontroller
int write_switch_reg(int num_bytes, int page_addr, int reg_addr, int data)
{	int err;

	//make sure interface not busy
	err = wait_for_done_bit();
	if(err != SUCCESS)
		return err;

	//Write the num_bytes
	err = spi_ind_reg_write(IND_NUM_BYTES, num_bytes);
	if(err != SUCCESS)
		return err;


	//Write the page address
	err = spi_ind_reg_write(IND_PAGE_ADDR, page_addr);
	if(err != SUCCESS)
		return err;
	
	//Write the reg address
	err = spi_ind_reg_write(IND_REG_ADDR, reg_addr);
	if(err != SUCCESS)
		return err;


	//write the data reg
	err = spi_ind_reg_write(IND_DATA, data);
	if(err != SUCCESS)
		return err;


	//Write the target register selecting MCU as the target
	err = spi_ind_reg_write(IND_TARGET_ADDR, SWITCH_TARGET | WRITE_OP);
	if(err != SUCCESS)
		return err;
	

	DELAY_SWITCH_ACCESS
	
	
	//wait for done bit to be set
	err = wait_for_done_bit();
	if(err != SUCCESS)
		return err;
	
	
    	
	return SUCCESS;
}




////////////////////////////
////////////////////////////
//Toggle reset pins, check d500e_mcu_register_map.doc Register 0x05
//for the bit descriptions.
int play_reset(void)
{	int err, mod, old;

	while(1)
	{
		printf("Enter reset bit(0-6).\n");
		scanf("%d", &mod);

		mod = mod % 7;
		
		//First save register contents
		err = read_mcu_reg(RESET_PINS_ADDR, &old);
		if(err != SUCCESS)
			return err;

		
		//Clear the bit
		err = write_mcu_reg(RESET_PINS_ADDR, old & (~(1 << mod)));
		if(err != SUCCESS)
			return err;

		//Wait a little before setting
		usleep(1000);

		//Set the bit
		err = write_mcu_reg(RESET_PINS_ADDR, old | (1 << mod));
		if(err != SUCCESS)
			return err;

	}	
	return SUCCESS;
  	
}

////////////////////////////
////////////////////////////
//Toggle reload pins of the modules, check d500e_mcu_register_map.doc Register 0x06
//for the bit descriptions.
int play_reload(void)
{	int err, mod, old;

	while(1)
	{
		printf("Enter module reload bit(0-4).\n");
		scanf("%d", &mod);

		mod = mod % 5;
		
		//First save register contents
		err = read_mcu_reg(VOC_RELOAD_PINS_ADDR, &old);
		if(err != SUCCESS)
			return err;

		
		//Clear the bit
		err = write_mcu_reg(VOC_RELOAD_PINS_ADDR, old & (~(1 << mod)));
		if(err != SUCCESS)
			return err;

		//Wait a little before setting
		usleep(1000);

		//Set the bit
		err = write_mcu_reg(VOC_RELOAD_PINS_ADDR, old | (1 << mod));
		if(err != SUCCESS)
			return err;

	}	
  	return SUCCESS;
}

////////////////////////////
////////////////////////////
//Test Switch read/write access.
int play_switch(void)
{	int err, low_16, high_16, t, port;

	//Verify Switch read access by reading Device ID
	err = read_switch_reg(2, 2, 0x30, &low_16);
	if(err != SUCCESS)
		return err;

	err = read_switch_reg(2,2,0x32, &high_16);
	if(err != SUCCESS)
		return err;

	low_16 |= (high_16 << 16);
	if(low_16 != SWITCH_DEVICE_ID)
	{	printf("ERROR: Failed to read Switch Device ID, expecting 0x%x, read back 0x%x.\n", SWITCH_DEVICE_ID, low_16);	
		return ERR_SWITCH_DEV_ID;
	}
	else
		printf("SWITCH_DEVICE_ID = 0x%x.\n", low_16);




	//Verify Switch write access by disabling all the ports
	//by setting bit 11 of Page 10h-17h, adresses 0 and 1
	
	
	printf("Hit Enter to disable all the ports.\n\r");
	getchar();
	
	for(port = 0x10; port < 0x18; port++)
	{		
		//First save the old value
		err = read_switch_reg(2,port,0,&t);
		if(err != SUCCESS)
			return err;
	
		err = write_switch_reg(2,port,0,t | (1<<11));
		if(err != SUCCESS)
			return err;
	}


	printf("Visually confirm that all the switch ports are off(Green LEDs OFF).\n\r");
	printf("Hit Enter to re-enable the ports.\n\r");
	getchar();

	//Re-enable all the ports by clearing bit 11
	for(port = 0x10; port < 0x18; port++)
	{	//First save the old value
			
		//First save the old value
		err = read_switch_reg(2,port,0,&t);
		if(err != SUCCESS)
			return err;
		err = write_switch_reg(2,port,0,t & (~(1<<11)));
		if(err != SUCCESS)
			return err;
	}


	return SUCCESS;
}

////////////////////////////
////////////////////////////
//Read MCU and SWITCH ID forever to test register access
int read_loop(void)
{	int err, loop, low_16, high_16, t;
	
	
	loop = 0;
	while(1)
	{	if((++loop % 1000) == 0)
			printf("looped %d times.\n", loop);

	
		//Verify MCU access by reading Device ID
		err = read_mcu_reg(DEVICE_ID_ADDR, &t);
		if(err != SUCCESS)
			return err;
	
		if(t != MCU_DEVICE_ID)
		{	printf("ERROR: Failed to read MCU ID, expecting 0x%x, read back 0x%x.\n", MCU_DEVICE_ID, t);
			return ERR_MCU_DEV_ID;
		}
		
		
		//Verify Switch access by reading Device ID
		err = read_switch_reg(2, 2, 0x30, &low_16);
		if(err != SUCCESS)
			return err;
	
		err = read_switch_reg(2,2,0x32, &high_16);
		if(err != SUCCESS)
			return err;
	
		low_16 |= (high_16 << 16);
		if(low_16 != SWITCH_DEVICE_ID)
		{	printf("ERROR: Failed to read Switch Device ID, expecting 0x%x, read back 0x%x.\n", SWITCH_DEVICE_ID, low_16);	
			return ERR_SWITCH_DEV_ID;
		}
	}

}



////////////////////////////
////////////////////////////


int main(int argc, char ** argv)
{	int err, t, low_16, high_16;


	if (argc < 2) {
		printf("Usage: ./tg3_test <devname>\n");
		printf("\n");
		printf("eg: ./tg_test eth0\n");
		exit(1);
	}

	err=MakeConnection(argv[1]);
	if (err) {
		printf("ERROR: Failed to connect.\n");
		return err;
	}
	
	// !!!! This function must be called first to initialize pins. !!!
	err=init_spi_pins();
	if (err != SUCCESS)
	{	return err;
	}
	
	//Verify MCU access by reading Device ID
	err = read_mcu_reg(DEVICE_ID_ADDR, &t);
  	if(err != SUCCESS)
    		return err;
	
    	if(t != MCU_DEVICE_ID)
	{	printf("ERROR: Failed to read MCU ID, expecting 0x%x, read back 0x%x.\n", MCU_DEVICE_ID, t);
		return ERR_MCU_DEV_ID;
	}
	else
		printf("MCU_DEVICE_ID = 0x%x.\n", t);

    	
	//Verify Switch access by reading Device ID
	err = read_switch_reg(2, 2, 0x30, &low_16);
	if(err != SUCCESS)
		return err;

	err = read_switch_reg(2,2,0x32, &high_16);
	if(err != SUCCESS)
		return err;

	low_16 |= (high_16 << 16);
	if(low_16 != SWITCH_DEVICE_ID)
	{	printf("ERROR: Failed to read Switch Device ID, expecting 0x%x, read back 0x%x.\n", SWITCH_DEVICE_ID, low_16);	
		return ERR_SWITCH_DEV_ID;
	}
	else
		printf("SWITCH_DEVICE_ID = 0x%x.\n", low_16);



    	//Read Board Revision
	err = read_mcu_reg(BOARD_REV_ADDR, &t);
	if(err != SUCCESS)
		return err;
		
	printf("BOARD_REV = %u.%u.\n", t>>8, t & 0xff);


    	//Read Firmware Version
	err = read_mcu_reg(FIRMWARE_VERSION_ADDR, &t);
	if(err != SUCCESS)
		return err;

   	printf("FIRMWARE_VERSION = %u.%u.\n", t>>8, t & 0xff);

	//Read Board Temperature(lower right corner of Switch Heatsink)
	err = read_mcu_reg(BOARD_TEMP_C_ADDR, &t);
	if(err != SUCCESS)
		return err;

	printf("BOARD TEMP C = %d.\n", t);


	//Read Fan Speed, 0 = off
	err = read_mcu_reg(FAN_SPEED_ADDR, &t);
	if(err != SUCCESS)
		return err;

	printf("FAN SPEED(0-10) = %d.\n", t);

	printf("\n\n\n");
	
	//play_reset();
	//play_reload();
	//play_switch();
	//read_loop();

	return 0;
}



